﻿package com.actionsnippet.qbox{
	
	import flash.display.*
	import flash.events.*;
	//import flash.geom.*;
	
	import Box2D.Dynamics.*;
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import Box2D.Dynamics.Joints.*;
	
	public class QuickBox2D extends EventDispatcher{
		
		public var worldAABB:b2AABB;
		public var gravity:b2Vec2;
		public var w:b2World;
		
		public var iterations:int = 20;
        public var timeStep:Number = 1 / 30;
		public var debug:Boolean;
		public var main:MovieClip;
		 
		private var mouseJoint:b2MouseJoint;
		private var mousePVec:b2Vec2 = new b2Vec2();
        
		public function QuickBox2D(main:MovieClip, param:Object=null){
			this.main = main;
			init(param);
		}
		
		private function init(p:Object=null):void{
			var defaults:Object = {gravityX:0.0, gravityY:20.0, iterations: iterations, timeStep: timeStep, bounds: [-100, -100, 100, 100], debug:false};
			 
			 if (p == null){
				 p = new Object();
			 }
			 
				for (var key:String in defaults){
					if (p[key] == null){
					  p[key] = defaults[key];
					}
				}
			 
			 debug = p.debug;
			 if (debug){
				main.graphics.beginFill(0x333333);
				main.graphics.drawRect(0,0, main.stage.stageWidth, main.stage.stageHeight);
			 }
			
			worldAABB  = new b2AABB();
			worldAABB.lowerBound.Set(p.bounds[0], p.bounds[1])
			worldAABB.upperBound.Set(p.bounds[2], p.bounds[3]);
			
			gravity = new b2Vec2(p.gravityX, p.gravityY);
			
			w=new b2World(worldAABB, gravity, true);
			
			if (debug){
				var dbgDraw:b2DebugDraw = new b2DebugDraw();
				var dbgSprite:Sprite = new Sprite();
				main.addChild(dbgSprite);
				dbgDraw.m_sprite = dbgSprite;
				dbgDraw.m_drawScale = 30.0;
				dbgDraw.m_fillAlpha = .5;
				dbgDraw.m_alpha = .5;
				dbgDraw.m_lineThickness = 1.0;
				dbgDraw.m_drawFlags = b2DebugDraw.e_shapeBit | b2DebugDraw.e_jointBit;//0xFFFFFFFF;
				w.SetDebugDraw(dbgDraw);
			}
			
		}
		
		public function grid():void{
			main.graphics.clear();
			main.graphics.lineStyle(0, 0xCCCCCC);
			for (var i:int = 0; i<main.stage.stageWidth; i+=30){
				main.graphics.moveTo(i, 0);
				main.graphics.lineTo(i, main.stage.stageHeight);
			}
			for (i = 0; i<main.stage.stageHeight; i+=30){
				main.graphics.moveTo(0, i);
				main.graphics.lineTo(main.stage.stageWidth, i);
			}
		}
		
		public function createStageWalls(p:Object=null):void{
			var defaults:Object = {lineColor:0x000000, lineAlpha:1, fillColor:0xCCCCCC, fillAlpha:1};
			 if (p == null){
				 p = new Object();
			 }
			 
			for (var key:String in defaults){
			      if (p[key] == null){
				     p[key] = defaults[key];
				  }
			 }
			var sw:Number = main.stage.stageWidth / 30;
			var sh:Number = main.stage.stageHeight / 30;
			
			 addBox({x:sw / 2, y:sh, width:sw - 1, height:1,  density:.0, lineColor: p.lineColor, lineAlpha: p.lineAlpha, fillColor: p.fillColor, fillAlpha: p.fillAllpha});
			 addBox({x:sw / 2, y:0, width:sw - 1, height:1,  density:.0, lineColor: p.lineColor, lineAlpha: p.lineAlpha, fillColor: p.fillColor, fillAlpha: p.fillAllpha});
			 addBox({x:0, y:sh / 2, width:1, height:sh ,  density:.0, lineColor: p.lineColor, lineAlpha: p.lineAlpha, fillColor: p.fillColor, fillAlpha: p.fillAllpha});
			 addBox({x:sw, y:sh / 2, width:1, height:sh ,  density:.0, lineColor: p.lineColor, lineAlpha: p.lineAlpha, fillColor: p.fillColor, fillAlpha: p.fillAllpha});
		}
		
		public function traceMouse():void{
			main.stage.addEventListener(MouseEvent.MOUSE_DOWN, onDown);
			main.stage.addEventListener(MouseEvent.MOUSE_UP, onUp);
		}
		
		private function onDown(evt:MouseEvent):void {
			main.addEventListener(Event.ENTER_FRAME, onTraceMouse);
		}
		private function onTraceMouse(evt:Event):void{
			trace("Mouse: ", (main.mouseX / 30).toFixed(2), (main.mouseY/30).toFixed(2));
		}
		private function onUp(evt:MouseEvent):void{
			main.removeEventListener(Event.ENTER_FRAME, onTraceMouse);
		}
		
		public function start():void{
			main.addEventListener(Event.ENTER_FRAME, onRender);
		}
		public function stop():void{
			main.removeEventListener(Event.ENTER_FRAME, onRender);
		}
		
		public function mouseDrag():void{
			main.stage.addEventListener(MouseEvent.MOUSE_DOWN, createMouse);
			main.stage.addEventListener(MouseEvent.MOUSE_UP, destroyMouse);
		}
		public function destroy():void{
			// additional clean up
			main.stage.removeEventListener(MouseEvent.MOUSE_DOWN, createMouse);
			main.stage.removeEventListener(MouseEvent.MOUSE_UP, destroyMouse);
			main.addEventListener(Event.ENTER_FRAME, onRender);
			main.stage.removeEventListener(MouseEvent.MOUSE_DOWN, onDown);
			main.stage.removeEventListener(MouseEvent.MOUSE_UP, onUp);
		}
		
		protected function onRender(evt:Event):void{
			w.Step(timeStep, iterations);
			
		    updateMouse();
			updateBodies();
			
			dispatchEvent(new Event("render"));
		}
		
		protected function updateBodies():void{
			 for (var bb:b2Body = w.m_bodyList; bb; bb = bb.m_next) {
				
				if (bb.m_userData is Sprite) {
					bb.m_userData.x=bb.GetPosition().x*30;
					bb.m_userData.y=bb.GetPosition().y*30;
					
					bb.m_userData.rotation = bb.GetAngle() * (180 / Math.PI);
				}else{
					 // render joints etc...
				}
	         } 
		}
		
		public function addPoly(params:Object):QuickObject{
			return create("poly", params);
		}
		public function addBox(params:Object):QuickObject{
			return create("box", params);
		}
		public function addCircle(params:Object):QuickObject {
			return create("circle", params);
		}
		public function addJoint(params:Object):QuickObject {
			return create("joint", params);
		}
		
		public function create(type:String, params:Object):QuickObject{
			if (type == "box"){
				return new BoxObject(this, params);
			}else if (type == "circle"){
				return new CircleObject(this, params);
			}else if (type == "poly"){
				return new PolyObject(this, params);
			}else  if (type == "joint"){
				return new JointObject(this, params);
			}else{
				throw new Error("QuickBox2D: Sorry, there is no QuickObject subclass for " + type + " types.");
			}
			return null;
		}
		//
		// -- mouse drag stuff (from the testbed code)
		//
		private function updateMouse():void{
			if (mouseJoint) {
				 
				var p2:b2Vec2=new b2Vec2(main.mouseX/30 ,main.mouseY/30);
				mouseJoint.SetTarget(p2);
			}
		}
		private function createMouse(evt:MouseEvent):void {
			var body:b2Body=GetBodyAtMouse();
			if (body) {
				var mouseJointDef:b2MouseJointDef=new b2MouseJointDef();
				mouseJointDef.body1=w.GetGroundBody();
				mouseJointDef.body2=body;
				mouseJointDef.target.Set(main.mouseX/30, main.mouseY/30);
				mouseJointDef.maxForce=3000;
				mouseJointDef.timeStep=timeStep;
				mouseJoint=w.CreateJoint(mouseJointDef) as b2MouseJoint;
			}
		}
		private function destroyMouse(evt:MouseEvent):void {
			if (mouseJoint) {
				w.DestroyJoint(mouseJoint);
				mouseJoint=null;
			}
		}
		private function GetBodyAtMouse(includeStatic:Boolean=false):b2Body {
			var mouseXWorldPhys = (main.mouseX)/30;
			var mouseYWorldPhys = (main.mouseY)/30;
			mousePVec.Set(mouseXWorldPhys, mouseYWorldPhys);
			var aabb:b2AABB = new b2AABB();
			aabb.lowerBound.Set(mouseXWorldPhys - 0.001, mouseYWorldPhys - 0.001);
			aabb.upperBound.Set(mouseXWorldPhys + 0.001, mouseYWorldPhys + 0.001);
			var k_maxCount:int=10;
			var shapes:Array = new Array();
			var count:int=w.Query(aabb,shapes,k_maxCount);
			var body:b2Body=null;
			for (var i:int = 0; i < count; ++i) {
				if (shapes[i].GetBody().IsStatic()==false||includeStatic) {
					var tShape:b2Shape=shapes[i] as b2Shape;
					var inside:Boolean=tShape.TestPoint(tShape.GetBody().GetXForm(),mousePVec);
					if (inside) {
						body=tShape.GetBody();
						break;
					}
				}
			}
			return body;
		}
	}
}