﻿package com.actionsnippet.qbox{

	import flash.display.*;
	import flash.events.*;
	import Box2D.Dynamics.*;
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import Box2D.Dynamics.Joints.*;

	public class QuickObject {

		public var bodyDef:b2BodyDef;
		public var body:b2Body;
		public var joint:b2DistanceJointDef;
		
		protected var defaults:Object;
		protected var params:Object;
		protected var type:String;
		
		protected var w:b2World;
		protected var qbox:QuickBox2D;
		
		
        public function get userData():*{
			return bodyDef.userData;
		}
		public function QuickObject(qbox:QuickBox2D, params:Object=null) {
			init(qbox, params);
		}

        // template method
		private final function init(qbox:QuickBox2D, params:Object=null):void {
			this.qbox = qbox;
			this.params=params;
			
            defineDefaults();
			setDefaults();

            // make sure we aren't a joint
            if (params.vecA == null){
			  setupBodyDef(params);
			}

			this.w  = qbox.w;
			
			// template hook
			 build();
			if (!qbox.debug){
				  // make sure we aren't a joint
				 if (params.vecA == null){
				   qbox.main.addChild(bodyDef.userData);
				 }
			}
			 
			   // make sure we aren't a joint
			 if (body != null){
				if (params.mass != null){
					var m:b2MassData = new b2MassData();
					m.mass = params.mass;
					body.SetMass(m);
				}else{
					body.SetMassFromShapes();
				}
			 }
		}

        private function setupBodyDef(p:Object):void{
			bodyDef = new b2BodyDef();
			bodyDef.position.x = p.x;
			bodyDef.position.y = p.y;
			bodyDef.angle = p.angle;
			bodyDef.linearDamping = p.linearDamping;
			bodyDef.angularDamping = p.angularDamping;
			bodyDef.fixedRotation = p.fixedRotation;
			bodyDef.isBullet = p.isBullet;
		}
        private function defineDefaults():void{
			defaults = {x:3, y:3, linearDamping:0, angularDamping:0, isBullet:true,
			                   fixedRotation:false,
		                       density:1.0, friction:0.5, restitution:0.2, angle:0.0, 
						       lineColor:0x000000, lineAlpha:1, 
						       fillColor:0xCCCCCC, fillAlpha:1} 
			// template hook
			defaultParams(defaults);
		}
   
        // hook for Object Specific params
		protected function defaultParams(p:Object):void{}
		
		// hook for box2D initialization
		protected function build():void {
			trace("You must override the build method of QuickObject");
		}

		private function setDefaults():void {
			if (params==null) {
				params = new Object();
			}
			for (var key:String in defaults) {
				if (params[key]==null) {
					params[key]=defaults[key];
				}
			}
		}
	}
}